//
//  FirstViewController.m
//  epuck2-ble-monitor
//
//  Created by Stefano Morgani on 01.05.19.
//  Copyright (c) 2019 GCtronic. All rights reserved.
//

#import "FirstViewController.h"

@interface FirstViewController ()

@end

@implementation FirstViewController

@synthesize btnConnect;
@synthesize scanResult;

- (void)viewDidLoad {
    
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    
    rawSensorViewController = (RawSensorsViewController*)[self.tabBarController.viewControllers objectAtIndex:1];
    
    if([[Robot sharedInstance] getConnectionState] == STATE_NOT_CONNECTED) {
        [btnConnect setTitle:@"Search" forState:UIControlStateNormal];
    } else if([[Robot sharedInstance] getConnectionState] == STATE_CONNECTING) {
        [btnConnect setTitle:@"Connecting" forState:UIControlStateNormal];
    } else {
        [btnConnect setTitle:@"Disconnect" forState:UIControlStateNormal];
    }
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateBleDevice:) name:@"discoverUpdateBleDeviceNotification" object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(addBleDevice:) name:@"discoverAddBleDeviceNotification" object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(bleOff:) name:@"bleOffNotification" object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectedBleDevice:) name:@"connectedBleDeviceNotification" object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(disconnectedBleDevice:) name:@"disconnectedBleDeviceNotification" object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(failToConnectBleDevice:) name:@"failToConnectBleDeviceNotification" object:nil];
    
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)updateBleDevice:(NSNotification*)notification {
    //NSInteger ind = notification.userInfo[@"index"];
    NSInteger ind = [[[notification userInfo] valueForKey:@"index"] intValue];
    
    [scanResult reloadRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:ind inSection:0]] withRowAnimation:UITableViewRowAnimationNone];
}

- (void)addBleDevice:(NSNotification*)notification {
    [scanResult insertRowsAtIndexPaths:[NSArray arrayWithObject:[NSIndexPath indexPathForRow:[[BleManager sharedInstance]peripheralsCount]-1 inSection:0]] withRowAnimation:UITableViewRowAnimationNone];
}

- (void)bleOff:(NSNotification*)notification {
    [scanResult reloadData];
    [btnConnect setTitle:@"Search" forState:UIControlStateNormal];
}

- (void)connectedBleDevice:(NSNotification*)notification {
    [btnConnect setTitle:@"Disconnect" forState:UIControlStateNormal];
    [[rawSensorViewController tabBarItem] setEnabled:true];
    
    // Start the RSSI thread as soon as at least one device is connected.
    //rssiTimer = [NSTimer scheduledTimerWithTimeInterval:0.2 target:self selector:@selector(rssiTask) userInfo:nil repeats:YES];
}

- (void)disconnectedBleDevice:(NSNotification*)notification {
    if([[Robot sharedInstance] isReconnectionEnabled]) {
        [btnConnect setTitle:@"Connecting" forState:UIControlStateNormal];
    } else {
        [btnConnect setTitle:@"Search" forState:UIControlStateNormal];
    }
    
    [[rawSensorViewController tabBarItem] setEnabled:false];
    //[rssiTimer invalidate];
}

- (void)failToConnectBleDevice:(NSNotification*)notification {
    if([[Robot sharedInstance] isReconnectionEnabled]) {
        [btnConnect setTitle:@"Connecting" forState:UIControlStateNormal];
    } else {
        [btnConnect setTitle:@"Search" forState:UIControlStateNormal];
    }
}

- (IBAction)btnConnectPressed:(UIButton *)sender {
    if([[BleManager sharedInstance] bleIsOff]) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Bluetooth OFF" message:@"Please turn on Bluetooth..." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        return;
    }
    
    if([[[btnConnect titleLabel] text] isEqualToString:@"Search"]) {
        NSLog(@"No peripheral connected, init scanning...");
        [[BleManager sharedInstance] clearDiscoveredPeripherals];
        [[BleManager sharedInstance] startScanning];
        [scanResult reloadData];
    } else if([[[btnConnect titleLabel] text] isEqualToString:@"Connecting"]) {
        // Do nothing
    } else if([[[btnConnect titleLabel] text] isEqualToString:@"Disconnect"]) {
        [[Robot sharedInstance] setReconnectionEnabled:false];
        [[BleManager sharedInstance] disconnectPeripheral];
    }

}

//-(void)updateRssiLabel {
//
//}
//
//-(void)rssiTask {
//
//    NSNumber *RSSI;
//
//    NSLog(@"rssi timer running");
//
//    //if(isGettingAccValueFlag == 0) {
//    if(![[Robot sharedInstance] isDisconnected]) {
//        [blePeripheral readRSSI];
//        RSSI = blePeripheral.RSSI;
//        bleDevRssiString = [NSString stringWithFormat:@"%d", [RSSI intValue]];
//        [self performSelectorOnMainThread:@selector(updateRssiLabel) withObject:nil waitUntilDone:NO];
//        NSLog(@"ble device rssi = %d", [RSSI intValue]);
//    } else {
//        bleDevRssiString = [NSString stringWithFormat:@"%@", @"not connected"];
//        [self performSelectorOnMainThread:@selector(updateRssiLabel) withObject:nil waitUntilDone:NO];
//    }
//
//    //}
//
//}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [[BleManager sharedInstance] peripheralsCount];
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"foundDevice";
    int16_t signalStrength = 0;
    
    UITableViewCell *cell = [scanResult dequeueReusableCellWithIdentifier:CellIdentifier];
    
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    }
    
    // Configure the cell.
    if([[BleManager sharedInstance] getPeripheralRssi:[indexPath row]] > 0) { // Sometimes we get 127 when the device is far away.
        //cell.textLabel.text = [NSString stringWithFormat:@"%d - %@ (%d)", 0, [[BleManager sharedInstance] getPeripheralName:[indexPath row]], [[BleManager sharedInstance] getPeripheralRssi:[indexPath row]]];
    } else {
        signalStrength = 100+[[BleManager sharedInstance] getPeripheralRssi:[indexPath row]];
        //cell.textLabel.text = [NSString stringWithFormat:@"%d - %@ (%d)", 100+[[BleManager sharedInstance] getPeripheralRssi:[indexPath row]], [[BleManager sharedInstance] getPeripheralName:[indexPath row]], [[BleManager sharedInstance] getPeripheralRssi:[indexPath row]]];
    }
    cell.textLabel.text = [NSString stringWithFormat:@"%@", [[BleManager sharedInstance] getPeripheralName:[indexPath row]]];
    
    cell.detailTextLabel.text = [NSString stringWithFormat:@"RSSI %d", [[BleManager sharedInstance] getPeripheralRssi:[indexPath row]]];
    
    
    if(signalStrength > 65) {
        cell.imageView.image = [UIImage imageNamed:@"signal_strength_100"];
    } else if(signalStrength > 45) {
        cell.imageView.image = [UIImage imageNamed:@"signal_strength_75"];
    } else if(signalStrength > 25) {
        cell.imageView.image = [UIImage imageNamed:@"signal_strength_50"];
    } else {
        cell.imageView.image = [UIImage imageNamed:@"signal_strength_25"];
    }
    
    
    return cell;
}

/*
 user selected row
 stop scanner
 connect peripheral for service search
 */
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [[Robot sharedInstance] setConnecting];
    [[Robot sharedInstance] setReconnectionEnabled:true];
    
    [[BleManager sharedInstance] setCurrentPeripheral:[indexPath row]];
    [[BleManager sharedInstance] connectToPeripheral];
    
    [btnConnect setTitle:@"Connecting" forState:UIControlStateNormal];
}

@end
