//
//  Robot.m
//  epuck2-ble-monitor
//
//  Created by Stefano Morgani on 01.05.19.
//  Copyright (c) 2019 GCtronic. All rights reserved.
//

#import "Robot.h"

//BOOL logEnabled;    // Declare it here to be able to access them from "class methods" (static methods).
//NSDate *startTime;
//double rxTimeSeconds;
//double rxTimeFreqSeconds;
//NSString *logFilePath;

@implementation Robot

+ (id)sharedInstance {
    static Robot *_sharedInstance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedInstance = [[self alloc] init];
    });
    return _sharedInstance;
}

- (id)init {
    if (self = [super init]) {
        connectionState = STATE_NOT_CONNECTED;
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(writeError:) name:@"writeErrorNotification" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(writeOk:) name:@"writeOkNotification" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateValueForCharacteristic:) name:@"updateValueForCharacteristicNotification" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(bleOff:) name:@"bleOffNotification" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectedBleDevice:) name:@"connectedBleDeviceNotification" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(disconnectedBleDevice:) name:@"disconnectedBleDeviceNotification" object:nil];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(failToConnectBleDevice:) name:@"failToConnectBleDeviceNotification" object:nil];
    }
    return self;
}

- (void)writeError:(NSNotification*)notification {
    writeResponseReceived = -1;
}

- (void)writeOk:(NSNotification*)notification {
    writeResponseReceived = 1;
}

- (void)updateValueForCharacteristic:(NSNotification*)notification {
    
    if([[BleManager sharedInstance]getRxLength] > 0) {
        [[BleManager sharedInstance] getRxBytes:bytePtr];
        
        for(int i=0; i<8; i++) {
            proxValues[i] = bytePtr[i*2] + bytePtr[i*2+1]*256;
            //NSLog(@"prox[%d] = %d", i, proxValues[i]);
        }
        
        // Get the packet time based on the phone time.
        rxTimeSeconds += [[NSDate date] timeIntervalSinceDate:startTime];
        rxTimeFreqSeconds += [[NSDate date] timeIntervalSinceDate:startTime];
        startTime = [NSDate date];
        rxFreqNumPackets++;
        if(rxFreqNumPackets == 10) {
            rxFreqNumPackets = 0;
            rxFreq = 10/rxTimeFreqSeconds;
            //NSLog(@"rxTimeSeconds = %f, delta = %f, freq = %d", rxTimeSeconds, rxTimeFreqSeconds, rxFreq);
            rxTimeFreqSeconds = 0;
        }
        
        [[NSNotificationCenter defaultCenter] postNotificationName:@"sensorsUpdatedNotification" object:self userInfo:nil];
        
        if(logEnabled) {
            [self appendData];
        }
        
    } else {
        NSLog(@"Empty rx data");
        [[BleManager sharedInstance] disconnectPeripheral];
    }
}

- (void)bleOff:(NSNotification*)notification {
    [self setConnected:FALSE];
}

- (void)connectedBleDevice:(NSNotification*)notification {
    [self setConnected:TRUE];
}

- (void)disconnectedBleDevice:(NSNotification*)notification {
    if([[Robot sharedInstance] isReconnectionEnabled]) {
        [[Robot sharedInstance] setConnecting];
        [[BleManager sharedInstance] connectToPeripheral];
    } else {
        [[Robot sharedInstance] setConnected:FALSE];
        [[BleManager sharedInstance] freeCurrentPeripheral];
    }
}

- (void)failToConnectBleDevice:(NSNotification*)notification {
    if([[Robot sharedInstance] isReconnectionEnabled]) {
        [[Robot sharedInstance] setConnecting];
        [[BleManager sharedInstance] connectToPeripheral];
    } else {
        [[Robot sharedInstance] setConnected:FALSE];
        [[BleManager sharedInstance] freeCurrentPeripheral];
    }
}

- (void)resetVariables {
    startTime = [NSDate date];
    rxTimeSeconds = 0;
    rxFreq = 0;
}

- (void)setConnecting {
    connectionState = STATE_CONNECTING;
}

- (void)setConnected:(BOOL)state {
    if(state) {
        connectionState = STATE_CONNECTED;
        robotName = [[BleManager sharedInstance]getCurrentPeripheralName];
        [self resetVariables];
    } else {
        connectionState = STATE_NOT_CONNECTED;
        robotName = nil;
    }
}

- (NSString*)getRobotName {
    return robotName;
}

- (uint8_t)getConnectionState {
    return connectionState;
}

- (void)setReconnectionEnabled:(BOOL)value {
    reconnectEnabled = value;
}

- (BOOL)isReconnectionEnabled {
    return reconnectEnabled;
}

- (BOOL)isDisconnected {
    if(connectionState == STATE_NOT_CONNECTED) {
        return true;
    } else {
        return false;
    }
}

- (void)setMotorLeft:(int) left andRight: (int) right {
    cmdToBleDev[0] = left&0xFF;
    cmdToBleDev[1] = left>>8;
    cmdToBleDev[2] = right&0xFF;
    cmdToBleDev[3] = right>>8;
    writeResponseReceived = 0;
    [[BleManager sharedInstance] writeArray:cmdToBleDev withResponse:FALSE];
}

- (uint16_t)getProxValue:(int)index {
    if(index < 8) {
        return proxValues[index];
    } else {
        return 0;
    }
}

- (uint8_t)getRxFrequency {
    return rxFreq;
}

- (NSString *)currentDateandTime {
    NSDate *today = [NSDate date];
    NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
    [dateFormat setDateFormat:@"yyyy-MM-dd 'at' HH.mm.ss.SSS"];
    NSString *dateString = [dateFormat stringFromDate:today];
    return dateString;
}

- (NSString *)dataFilePath {
    NSString *date = [self currentDateandTime];
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    return[documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"log_%@.csv",date]];
}

- (void)startLog {
    //NSLog(@"start logging");
    logFilePath = [self dataFilePath];
    NSLog(@"file path = %@", logFilePath);
    logEnabled = true;
}

- (void)stopLog {
    //NSLog(@"stop logging");
    logEnabled = false;
}

- (BOOL)isLogEnabled {
    return logEnabled;
}

- (void)appendData {
    
    //NSLog(@"file path = %@", logFilePath);
    
    NSMutableString *writeString = [NSMutableString stringWithCapacity:0]; //don't worry about the capacity, it will expand as necessary
    
    // If the file doesn't exist then create the file and header of the csv (titles and comments).
    if (![[NSFileManager defaultManager] fileExistsAtPath:logFilePath]) {
        [[NSFileManager defaultManager] createFileAtPath:logFilePath contents:nil attributes:nil];
        
        // Header.
        [writeString appendString:[NSString stringWithFormat:@"rx time (sec);prox0;prox1;prox2;prox3;prox4;prox5;prox6;prox7\n"]];
        
    }
    
    [writeString appendString:[NSString stringWithFormat:@"%.3f;%d;%d;%d;%d;%d;%d;%d;%d\n", rxTimeSeconds, proxValues[0], proxValues[1], proxValues[2], proxValues[3], proxValues[4], proxValues[5], proxValues[6], proxValues[7]]];
    
    //NSLog(@"writeString :%@",writeString);
    
    NSFileHandle *handle;
    handle = [NSFileHandle fileHandleForWritingAtPath: logFilePath]; //say to handle where's the file fo write
    [handle truncateFileAtOffset:[handle seekToEndOfFile]]; //position handle cursor to the end of file
    [handle writeData:[writeString dataUsingEncoding:NSUTF8StringEncoding]];
    
}

@end
